package Plugins::LDIFSync;

use Net::LDAP::LDIF;
use Net::LDAP::Entry;
use Config::General;
use MIME::Base64;
use XML::Simple;

$VERSION  = 1.0;
my $plugin_name = 'LDIFSync';
$this_pkg = (caller(0))[0];

my $conf = new Config::General(
    -ConfigFile      => $config::plugins_dir . '/'.$plugin_name.'.conf',
    -InterPolateVars => TRUE,
    -AutoLaunder     => TRUE
);
my %cfg = $conf->getall;

$config::ldif_sync_file_prefix = $config::log_dir . '/' . $cfg{'ldif_sync_file_prefix'};
$config::ldif_sync_file_ext    = $cfg{'ldif_sync_file_ext'};
$config::ldif_dump_file_prefix = $config::log_dir . '/' . $cfg{'ldif_dump_file_prefix'};
$config::ldif_dump_file_ext    = $cfg{'ldif_dump_file_ext'};
$config::logrotate_file_prefix = $config::log_dir . '/' . $cfg{'logrotate_file_prefix'};
$config::logrotate_file_ext    = $cfg{'logrotate_file_ext'};
@valid_ips                     = keys %{ $cfg{'valid_ip'} };

AOSICFG::write_ws_log('-=- ' . $this_pkg . '::'.$plugin_name.': getting IP list: ' . join(", ", @valid_ips)) if $config::aosi_debug_level > 1;
AOSICFG::write_ws_log('-=- ' . $this_pkg . '::'.$plugin_name.': sync_file=' . $config::ldif_sync_file_prefix . '_<realm>' . $config::ldif_sync_file_ext)
  if $config::aosi_debug_level > 1;
AOSICFG::write_ws_log('-=- ' . $this_pkg . '::'.$plugin_name.': ldif_dump_file=' . $config::ldif_dump_file_prefix . '_<realm>' . $config::ldif_dump_file_ext)
  if $config::aosi_debug_level > 1;
AOSICFG::write_ws_log('-=- ' . $this_pkg . '::'.$plugin_name.': logrotate_file=' . $config::logrotate_file_prefix . '_<realm>' . $config::logrotate_file_ext)
  if $config::aosi_debug_level > 1;
AOSICFG::write_ws_log('-=- ' . $this_pkg . '::'.$plugin_name.': logrotate_exe_path=' . $cfg{'logrotate_exe_path'} . '; logrotate_exe=' . $cfg{'logrotate_exe'})
  if $config::aosi_debug_level > 1;

##################################################################
#  Check if IP is in the list
##################################################################
sub sync_is_valid {
    my ($in) = @_;
    my ($ip, $ok);

    $ok = 0;
    foreach $ip (@valid_ips) {
        chomp($ip);
        if ($ip eq $in) {
            $ok++;
        }
    }
    AOSICFG::write_ws_log('| Sync IP ' . $in . ' is valid: ' . $ok) if $config::aosi_debug_level > 2;
    return $ok;
}

##################################################################
#  Extraxt realm from base DN
##################################################################
sub base2realm {
    my ($base) = @_;
    my ($temp, $realm);

    ($realm) = split(/,/, $base);
    ($temp, $realm) = split(/=/, $realm);

    return $realm;
}

##################################################################
#  Write a logrotate config file
##################################################################
sub make_logrot {
    my ($base) = @_;
    my ($code, $msg, $file, $realm, $this_fun, $time_diff);
    my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks);
    $this_fun = (caller(0))[3];

    $realm = base2realm($base);
    $file  = $config::logrotate_file_prefix . "_" . $realm . $config::logrotate_file_ext;
    $out   = $file;
    $code  = 0;

    ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat($file);
    $time_diff = time - $mtime;
    AOSICFG::write_ws_log("|> $this_fun:mtime=$mtime;time-mtime=$time_diff;") if $config::aosi_debug_level > 2;

    if ((!-e $file) or ($time_diff > 604800)) {    # 604800 sec = 7 days
          AOSICFG::write_ws_log("|> $this_fun: creating file $file.") if $config::aosi_debug_level > 2;
          open(LOGR, ">$file") || do { $code = -11; $out = 'Cannot create ' . $file . '(' . $! . ')' };
          if ($code == 0) {
              print LOGR $config::ldif_sync_file_prefix . "_" . $realm . $config::ldif_sync_file_ext;
              #<<<
              print LOGR ' {
     rotate 5
     missingok
     notifempty
     create 0664 aosi aosi
}';
            #>>>
              close(LOGR);
          }
    }

    return $code, $out;
}

##################################################################
#  Dump whole LDAP
##################################################################
sub dumpAll {
      my ($ldap, $base) = @_;
      my ($this_fun, $mesg, $t1, $t2, $code, $mesg2, $val, $att, $ent, @entries);
      my ($out_error, $out_result, $out_code, $ldif, @values, $realm);
      $this_fun = (caller(0))[3];

      $realm = base2realm($base);
      AOSICFG::write_ws_log("|> $this_fun(in): base=$base; realm=$realm");
      $mesg = $ldap->bind($config::ldap_user, password => AOSICFG::get_pwd($config::ldap_pwd_file), version => 3);
      if ($mesg->code != 0) {
          AOSICFG::write_ws_log("|> $this_fun !!!$log: invalid admin password in " . $config::ldap_pwd_file);
          $out_result = 'Invalid admin password!';
          $out_code   = $mesg->code;
      }
      else {
          AOSICFG::write_ws_log("|> $this_fun(bind_as_admin): " . $mesg->code) if $config::aosi_debug_level > 2;
          %options = (
              base      => $base,
              filter    => '(objectclass=*)',
              timelimit => 300
          );
          $t1 = time;
          AOSICFG::write_ws_log("|> $this_fun:time1= " . $t1) if $config::aosi_debug_level > 2;
          $mesg2 = $ldap->search(%options);
          $t2 = time;
          AOSICFG::write_ws_log("|> $this_fun:time2= " . $t2) if $config::aosi_debug_level > 2;
          AOSICFG::write_ws_log("|> $this_fun:time2-time1= " . sprintf("%d", $t2 - $t1)) if $config::aosi_debug_level > 2;
          $code = $mesg2->code;

          if ($code == 0) {
              @entries = $mesg2->entries;
              AOSICFG::write_ws_log("|> $this_fun: #entries=" . $#entries) if $config::aosi_debug_level > 2;
              $ldif = Net::LDAP::LDIF->new(
                  $config::ldif_dump_file_prefix . "_" . $realm . $config::ldif_dump_file_ext, "w",
                  encode  => 'base64',
                  onerror => 'warn',
                  change  => 1
              );
              if ($ldif) {
                  if ($ldif->error()) {
                      AOSICFG::write_ws_log("|> $this_fun: LDIF error: " . $ldif->error()) if $config::aosi_debug_level > 2;
                      $out_result = 'LDIF error: ' . $ldif->error();
                      $out_code   = -24;
                  }
                  else {
                      foreach $ent (@entries) {
                          $ent->changetype('add');
                          $ldif->write_entry($ent);
                      }
                      $out_code = 0;
                  }
              }
              $ldif->done();
          }
          else {
              $out_result = $mesg2->error;
              $out_code   = $code;
          }
          $mesg = $ldap->unbind;
          my $t3 = time;
          AOSICFG::write_ws_log("|> $this_fun:time3= " . $t2) if $config::aosi_debug_level > 2;
          AOSICFG::write_ws_log("|> $this_fun:time3-time2= " . sprintf("%d", $t3 - $t2)) if $config::aosi_debug_level > 2;
      }
      AOSICFG::write_ws_log("|> $this_fun(out): out_code=$out_code;out_result=$out_result;");
      return $out_code, $out_result;
}

######################################################################
# Get LDIF for backup snyc purposess
######################################################################
# $type
# 1 = Get orig file info
# 2 = Get orig file and rotate
# 3 = Get rotated file info
# 4 = Get rotated file
# 5 = Get backup file info
# 6 = Backup all entries
# 7 = Get backup file
sub AOSI::ldapLDIFSync {
      my ($service, $user, $pwd, $base, $type) = @_;
      my ($hash,   $out_msg, $log,  $dn,       $ldap, $ldap_error, $dec_pwd, $mesg,  $error, $code, $dummy);
      my ($filter, @entries, $port, $hostname, $num,  $file,       $file1,   $file0, $ldif,  @ldif, $sys, $run);
      my ($dev,    $ino,     $mode, $nlink,    $uid,  $gid,        $rdev,    $size,  $atime, $mtime, $ctime, $blksize, $blocks);
      my ($logrotate_file);
      $log = "ldapLDIFSync";

      AOSICFG::write_ws_log("/ $log(in):user:$user;base:$base;type:$type;") if $config::aosi_debug_level > 0;
      AOSICFG::write_ws_log("| $log: IP:$AOSI::client_ip;")                 if $config::aosi_debug_level > 1;

      $realm = base2realm($base);
      $file0 = $config::ldif_sync_file_prefix . "_" . $realm . $config::ldif_sync_file_ext;
      $file1 = $config::ldif_sync_file_prefix . "_" . $realm . $config::ldif_sync_file_ext . '.1';
      $file2 = $config::ldif_dump_file_prefix . "_" . $realm . $config::ldif_dump_file_ext;
      if (($type == 1)) {
          $file = $file0;
      }
      elsif (($type >= 5) and ($type <= 7)) {
          $file = $file2;
      }
      else {
          $file = $file1;
      }
      AOSICFG::write_ws_log("| $log: LDIF_file=$file;") if $config::aosi_debug_level > 1;
      ($hostname, $port) = AOSICFG::parse_server($config::ldap_server);
      ($ldap, $ldap_error) = AOSICFG::ldap_init($hostname, $port);
      if ($user =~ /=/g) {
          $dn = $user . ',' . $base;
      }
      else {
          $dn = 'uid=' . $user . ',' . $base;
      }
      AOSICFG::write_ws_log("| $log: dn=$dn;") if $config::aosi_debug_level > 1;

      if (sync_is_valid($AOSI::client_ip)) {
          if ($ldap) {
              $dec_pwd = decode_base64($pwd);
              $mesg    = $ldap->bind($dn, password => $dec_pwd, version => 3);
              $code    = $mesg->code;
              $error   = $mesg->error;
              if ($code == 0) {
                  if (($type == 1) or ($type == 3) or ($type == 5)) {    # LDIF file info
                      ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat($file);
                      if ($size eq '') {
                          $size = 0;
                      }
                      $out_msg = '<attribute ldapname="size"><value>' . $size . '</value></attribute>';
                      $out_msg .= '<attribute ldapname="modify_time"><value>' . AOSI::t2t($mtime) . '</value></attribute>';
                      $out_msg .= '<attribute ldapname="access_time"><value>' . AOSI::t2t($atime) . '</value></attribute>';
                      $out_msg .= '<attribute ldapname="file"><value>' . $file . '</value></attribute>';
                  }
                  elsif ($type == 2) {                                   # Get LDIF file
                      $ENV{'PATH'} = $cfg{'logrotate_exe_path'};
                      ($code, $logrot_file) = make_logrot($base);
                      AOSICFG::write_ws_log("| $log: logrot_file=$logrot_file;code=$code;") if $config::aosi_debug_level > 2;

                      # Perltidy escape
                    #<<<
                    $run = $cfg{'logrotate_exe'} . ' -f -v ' . $logrot_file . ' -s '
                           . $config::log_dir . '/' . $cfg{'logrotate_status_prefix'} . "_" . $realm . ' 2>&1';
                    #>>>
                      $sys = qx($run);
                      AOSICFG::write_ws_log("| $log: run=$run;sys=$sys;") if $config::aosi_debug_level > 2;
                      if ($sys =~ /error/g) {
                          $code    = -20;
                          $out_msg = '<status code="' . $code . '">LDIF rotate error (' . $sys . ')!</status>';
                      }
                      elsif ($sys =~ /log does not need rotating/g) {
                          $out_msg .= '<attribute ldapname="size"><value>0</value></attribute>';
                          $out_msg .= '<attribute ldapname="warning"><value>LDIF is empty!</value></attribute>';
                      }
                      else {
                          open(LDIF, $file)
                            or do { $code = -21; $out_msg = '<status code="' . $code . '">Can not open file ' . $file . '</status>'; $error = $!; };
                          @ldif = <LDIF>;
                          close(LDIF);
                          $ldif = join("", @ldif);
                          AOSICFG::write_ws_log("| $log: ldif=$ldif;") if $config::aosi_debug_level > 3;
                          ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat($file);
                          $ldif = XML::Simple::escape_value($dummy, $ldif);
                          $out_msg = '<attribute ldapname="ldif"><value>' . $ldif . '</value></attribute>';
                          $out_msg .= '<attribute ldapname="size"><value>' . $size . '</value></attribute>';
                      }
                  }
                  elsif (($type == 4) or ($type == 7)) {
                      open(LDIF, $file)
                        or do { $code = -20; $out_msg = '<status code="' . $code . '">Can not open file ' . $file . '</status>'; $error = $!; };
                      if ($code == 0) {
                          @ldif = <LDIF>;
                          close(LDIF);
                          $ldif = join("", @ldif);
                          ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat($file);
                          $ldif = XML::Simple::escape_value($dummy, $ldif);
                          $out_msg = '<attribute ldapname="ldif"><value>' . $ldif . '</value></attribute>';
                          $out_msg .= '<attribute ldapname="size"><value>' . $size . '</value></attribute>';
                      }
                      close(LDIF);
                  }
                  elsif (($type == 6)) {
                      ($code, $error) = dumpAll($ldap, $base);
                      if ($code == 0) {
                          $dn = $config::ldap_user;
                          ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat($file);
                          $out_msg .= '<attribute ldapname="size"><value>' . $size . '</value></attribute>';
                          $error = '';
                      }
                      else {
                          $out_msg = '<status code="' . $code . '">' . $error . '</status>';
                      }
                  }
                  $mesg = $ldap->unbind;
              }
              else {
                  $out_msg = '<status code="' . $code . '">' . $error . '</status>';
              }
          }
          else {
              $code    = -18;
              $error   = $ldap_error;
              $out_msg = '<status code="' . $code . '">' . $error . '</status>';
          }
      }
      else {
          $code    = -23;
          $error   = 'You are not allowed to call this function!';
          $out_msg = '<status code="' . $code . '">' . $error . '</status>';
      }
      $hash->{'code'}  = $code;
      $hash->{'error'} = $error;
      $hash->{'result'} =
        '<?xml version="1.0"?><ldap action="' . $log . '" version="' . $config::aosi_version . '"><entry dn="' . $dn . '">' . $out_msg . '</entry></ldap>';

      AOSICFG::write_ws_log("\\ $log(out):" . AOSICFG::dump_hash($hash)) if $config::aosi_debug_level > 0;
      return AOSI::hash2data($hash, { 'code' => 'int' });
}

##################################################################
#  Fill entries
##################################################################
# 1 = afterAddAttribute
# 2 = afterModifyAttribute
# 3 = afterDeleteAttribute
# 4 = afterAddUser
# 5 = afterDeleteUser
sub hash2ent {
      my ($hash, $type) = @_;
      my ($this_fun, $entry, $dn, $entry_stg, $att, $val, $att_item);
      $this_fun = (caller(0))[3];

      AOSICFG::write_ws_log("|> $this_fun(in): type=$type;hash_keys=" . join(", ", keys %$hash)) if $config::aosi_debug_level > 2;
      $dn = $hash->{'dn'};
      AOSICFG::write_ws_log("|> $this_fun: dn=$dn;") if $config::aosi_debug_level > 2;
      $entry = Net::LDAP::Entry->new();
      $entry->dn($dn);
      if ($type <= 3) {    # Modify entry
          $entry->changetype('modify');
      }
      elsif ($type == 4) {    # Add entry
          $entry->changetype('add');
          $entry->add('objectClass' => 'person');
          $entry->add('objectClass' => 'organizationalPerson');
          $entry->add('objectClass' => 'inetOrgPerson');
          $entry->add('objectClass' => 'hrEduPerson');
          $entry->add('objectClass' => 'schacContactLocation');
      }
      elsif ($type == 5) {    # Delete entry
          $entry->changetype('delete');
          $entry->delete();
      }
      foreach $att_item (@{ $hash->{'attrs'} }) {
          if (ref($att_item) eq 'HASH') {
              $att = $att_item->{'att'};
              $val = $att_item->{'val'};
              if ($type == 1) {
                  $entry->add($att => $val);
              }
              elsif ($type == 2) {
                  $entry->replace($att => $val);
              }
              elsif ($type == 3) {
                  $entry->delete($att => $val);
              }
              elsif ($type == 4) {
                  $entry->add($att => $val);
              }
              AOSICFG::write_ws_log("|> $this_fun: $att = $val") if $config::aosi_debug_level > 2;
          }
      }
      return $entry;
}

##################################################################
#  Write to file
##################################################################
sub append_to_ldif {
      my ($entries, $type, $base_dn) = @_;
      my ($this_fun, $file, $dn, $ent, $path, $uid, $base);
      $this_fun = (caller(0))[3];

      AOSICFG::write_ws_log("|> $this_fun(in): type=$type;entries_keys=" . join(", ", keys %$entries)) if $config::aosi_debug_level > 2;

      ($uid, $base) = AOSICFG::get_uid_base($entries->{'dn'});
      if (cmp_dn($base, $base_dn)) {
          $file = $config::ldif_sync_file_prefix . "_" . base2realm($base) . $config::ldif_sync_file_ext;

          # Hacking paths for taint
          #$file =~ /\/([^;\&]+)/;
          #$file = $1;
          AOSICFG::write_ws_log("|> $this_fun: file2=$file;") if $config::aosi_debug_level > 2;
          if (!-e $file) {
              utime(time, time, $file);
          }
          $ldif = Net::LDAP::LDIF->new($file, "a", encode => 'base64', onerror => 'warn', change => 1);
          if ($ldif) {
              if ($ldif->error()) {
                  AOSICFG::write_ws_log("|> $this_fun: LDIF error: " . $ldif->error()) if $config::aosi_debug_level > 2;
              }
              else {
                  $ent = hash2ent($entries, $type);
                  $ldif->write_entry($ent);
                  $ldif->done();
              }
          }
          AOSICFG::write_ws_log("|> $this_fun(out): OK") if $config::aosi_debug_level > 2;
      }
      else {
          AOSICFG::write_ws_log("|> $this_fun(out): admin base dn ($base_dn) != entry base dn ($base)") if $config::aosi_debug_level > 2;
      }
}

##################################################################
#   beforeAddUser
##################################################################
sub beforeAddUser {
      my ($status_hash, $ldap, $base_dn, $entry_in, $user) = @_;
      my ($this_fun, $file, $msg, $code, $out_panic, $uid, $base);
      $this_fun  = (caller(0))[3];
      $msg       = $base_dn;
      $code      = 0;
      $out_panic = 0;

      AOSICFG::write_ws_log("|> $this_fun(in): base_dn=$base_dn;user=$user;entry_in=$entry_in;") if $config::aosi_debug_level > 1;
      ($uid, $base) = AOSICFG::get_uid_base($entry_in->{'dn'});
      AOSICFG::write_ws_log("|> $this_fun: base=$base") if $config::aosi_debug_level > 1;
      if (!cmp_dn($base, $base_dn)) {
          $code      = -10401;
          $msg       = "Admin user base DN ($base_dn) and entry base DN ($base) are different!";
          $out_panic = 1;
      }

      return $code, $msg, $out_panic;
}

##################################################################
#   afterAddUser
##################################################################
sub afterAddUser {
      my ($status_hash, $entry_in, $out_xml) = @_;
      my ($this_fun, $file, $msg, $code, $out_panic, $ip, $base);
      $this_fun  = (caller(0))[3];
      $msg       = '';
      $code      = 0;
      $out_panic = 0;

      AOSICFG::write_ws_log("|> $this_fun(in): \nentry_in=\n" . Plugins::dump_entry($entry_in) . "\nout_xml=\n" . $out_xml) if $config::aosi_debug_level > 1;
      $base = $status_hash->{$plugin_name}->{'before'}->{'msg'};
      AOSICFG::write_ws_log("|> $this_fun: base=$base") if $config::aosi_debug_level > 1;
      ($code, $msg) = Plugins::parse_ldap_xml($out_xml);
      if ($code == 0) {
          append_to_ldif($entry_in, 4, $base);
      }
      else {  # Main action will report the error anyway
          $code = 0;
      }
      AOSICFG::write_ws_log("|> $this_fun(out): code=$code;msg=$msg;panic=$out_panic;") if $config::aosi_debug_level > 1;
      return $code, $msg, $out_panic;
}

##################################################################
#   beforeDeleteUser
##################################################################
sub beforeDeleteUser {
      my ($status_hash, $ldap, $base_dn, $entry_in, $user) = @_;
      my ($this_fun, $file, $msg, $code, $out_panic, $uid, $base);
      $this_fun  = (caller(0))[3];
      $msg       = $base_dn;
      $code      = 0;
      $out_panic = 0;

      AOSICFG::write_ws_log("|> $this_fun(in): base=$base_dn;user=$user;entry_in=$entry_in;") if $config::aosi_debug_level > 1;
      ($uid, $base) = AOSICFG::get_uid_base($entry_in->{'dn'});
      AOSICFG::write_ws_log("|> $this_fun: base=$base") if $config::aosi_debug_level > 1;
      if (!cmp_dn($base, $base_dn)) {
          $code      = -10402;
          $msg       = "Admin user base DN ($base_dn) and entry base DN ($base) are different!";
          $out_panic = 1;
      }

      return $code, $msg, $out_panic;
}

##################################################################
#   afterDeleteUser
##################################################################
sub afterDeleteUser {
      my ($status_hash, $entry_in, $out_xml) = @_;
      my ($this_fun, $file, $msg, $code, $panic, $out_panic);
      $this_fun  = (caller(0))[3];
      $msg       = '';
      $code      = 0;
      $out_panic = 0;

      AOSICFG::write_ws_log("|> $this_fun(in): \nentry_in=\n" . Plugins::dump_entry($entry_in) . "\nout_xml=\n" . $out_xml) if $config::aosi_debug_level > 1;
      $base = $status_hash->{$plugin_name}->{'before'}->{'msg'};
      AOSICFG::write_ws_log("|> $this_fun: base=$base") if $config::aosi_debug_level > 1;
      ($code, $msg) = Plugins::parse_ldap_xml($out_xml);
      if ($code == 0) {
          append_to_ldif($entry_in, 5, $base);
      }
      else {  # Main action will report the error anyway
          $code = 0;
      }
      AOSICFG::write_ws_log("|> $this_fun(out): code=$code;msg=$msg;panic=$out_panic;") if $config::aosi_debug_level > 1;
      return $code, $msg, $out_panic;
}

##################################################################
#   before*Attribute
##################################################################
# Same function for all actions
# 1 = afterAddAttribute
# 2 = afterModifyAttribute
# 3 = afterDeleteAttribute
sub beforeChangeAttribute {
      my ($status_hash, $type, $ldap, $base_dn, $entry_in, $user, $pwd) = @_;
      my ($this_fun, $file, $out_panic, $uid, $base);
      $this_fun  = (caller(0))[3];
      $msg       = $base_dn;
      $code      = 0;
      $out_panic = 0;

      AOSICFG::write_ws_log("|> $this_fun(in): base=$base_dn;user=$user;entry_in=$entry_in;") if $config::aosi_debug_level > 1;
      ($uid, $base) = AOSICFG::get_uid_base($entry_in->{'dn'});
      AOSICFG::write_ws_log("|> $this_fun: base=$base") if $config::aosi_debug_level > 1;
      if (!cmp_dn($base, $base_dn)) {
          $code      = -10403;
          $msg       = "Admin user base DN ($base_dn) and entry base DN ($base) are different!";
          $out_panic = 1;
      }
      AOSICFG::write_ws_log("|> $this_fun(out): code=$code;msg=$msg;panic=$out_panic;") if $config::aosi_debug_level > 1;
      return $code, $msg, $out_panic;
}

##################################################################
#   after*Attribute
##################################################################
# Same function for all actions
# 1 = afterAddAttribute
# 2 = afterModifyAttribute
# 3 = afterDeleteAttribute
sub afterChangeAttribute {
      my ($status_hash, $type, $entry_in, $out_xml) = @_;
      my ($this_fun, $file, $out_panic);
      $this_fun  = (caller(0))[3];
      $msg       = '';
      $code      = 0;
      $out_panic = 0;

      AOSICFG::write_ws_log("|> $this_fun(in): \ntype=$type;\nentry_in=\n" . Plugins::dump_entry($entry_in) . "\nout_xml=\n" . $out_xml)
        if $config::aosi_debug_level > 1;
      $base = $status_hash->{$plugin_name}->{'before'}->{'msg'};
      AOSICFG::write_ws_log("|> $this_fun: base=$base") if $config::aosi_debug_level > 1;
      ($code, $msg) = Plugins::parse_ldap_xml($out_xml);
      if ($code == 0) {
          append_to_ldif($entry_in, $type, $base);
      }
      else {  # Main action will report the error anyway
          $code = 0;
      }
      AOSICFG::write_ws_log("|> $this_fun(out): code=$code;msg=$msg;panic=$out_panic;") if $config::aosi_debug_level > 1;
      return $code, $msg, $out_panic;
}

sub cmp_dn {
    my($a, $b) = @_;
    my(@A, @B, $i, $ok, $out);

    AOSICFG::write_ws_log("|> cmp_dn: a=$a; b=$b;") if $config::aosi_debug_level > 3;
    @A = split(/[,;]\s*/, $a);
    @B = split(/[,;]\s*/, $b);

    $ok = -1;
    foreach $i(0..$#A){
        if($A[$i] eq $B[$i]){
            $ok++;
        }
    }
    AOSICFG::write_ws_log("|> cmp_dn: ok=$ok;") if $config::aosi_debug_level > 3;

    $out = $ok == $#A ? 1 : 0;

    return $out;
}

1;
